  function [Q,grid] = adapttrapez(f,a,b,tol,maxlevel)

% function [Q,grid] = adapttrapez(f,a,b,tol)
%
% Zweck: approximiere das bestimmte Integral von f(x) von a bis b bis
%        auf die Toleranz tol mit adaptiver Trapez Quadratur und maximum
%        Verfeinerungs-Level maxlevel
%
% Parameters: f        ... die zu integrierende Funktion 
%             a,b      ... linke/rechte Integrations-Grenzen
%             tol      ... Toleranz
%             maxlevel ... Maximale Anzahl Verfeinerungen
%
% Returns: Q    ... Integral Approximation
%          grid ... das adaptive Gitter fuer Q
%
% Notes: Als Fehler-Schaetzer wird der in Bsp. (14) & (15) verwendet.
%

% Berechne erste Approx. mit Trapezegel ueber das ganze Intervall
  fa = f(a);
  fb = f(b);
  grid = [a,b];
  Q = 0.5*(b - a)*(fa + fb);

% Verfeinere die Approx. mit adaptiver Trapez Quadratur
  [Q,grid] = adapttrapeze(f,a,b,tol,fa,fb,Q,maxlevel,grid);

% Unique & sort das adaptive Gitter
  grid = sort(unique(grid));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  function [Q,grid] = adapttrapeze(f,a,b,tol,fa,fb,Q,level,grid)

% function adapttrapeze(f,a,b,tol,fa,fb,Q,level,grid)
%
% Zweck: Rekursiver Schritt fuer adaptive Trapez Quadratur. Diese Funktion
%        ist nur da um den User von dem muehsamen fa,fb,Q,... Inputs zu
%        befreien.
%
% Parameter: f        ... die zu integrierende Funktion
%            a,b      ... linke/rechte Integrations-Grenzen
%            tol      ... Toleranz
%            f{a,b}   ... Funktionauswertungen f(a) & f(b)
%            Q        ... derzeitige Approx. des Integrals mit der Simpson-
%                         Regel ueber [a,b]
%            level    ... derzeitiges Verfeinerungs level
%            grid     ... derzeitiges adaptives Gitter
%
% Returns: Q    ... Integral Approximation
%          grid ... das adaptive Gitter fuer Q
%
% Notes: Als Fehler-Schaetzer wird der in Bsp. (14) & (15) verwendet.
%

% Berechne Fehler-Schaetzer durch anwenden der Trapezegel auf halb-Intervalle
  m = 0.5*(a + b);
  grid = cat(2,grid,[m]);
  fm = f(m);
  Ql = 0.5*(m - a)*(fa + fm);
  Qr = 0.5*(b - m)*(fm + fb);
  E2 = abs(Q - (Ql + Qr))/3.;

% Ueberpruefe ob Toleranz erreicht
  if ( E2 < tol ) % Toleranz erreicht?
    Q = Ql + Qr;
  else % Verfeinere das Intervall falls moeglich
    if ( level == 0 ) % Maximale Verfeinerung erreicht!
      fprintf('Warnung: Maximale Verfeinerung erreicht!\n');
      Q = Ql + Qr;
    else % Verfeinere
      [Ql,gridl] = adapttrapeze(f,a,m,0.5*tol,fa,fm,Ql,level-1,grid);
      [Qr,gridr] = adapttrapeze(f,m,b,0.5*tol,fm,fb,Qr,level-1,grid);
      grid = cat(2,gridl,gridr);
      Q = Ql + Qr;
    end
  end

